@@ -10,11 +10,11 @@ As BioSTEAM objects are created, they are automatically registered. The `main_fl
 """
 import biosteam as bst
 from thermosteam.utils import Registry
-from ._digraph import (digraph_from_units,
-                       digraph_from_units_and_streams, 
-                       finalize_digraph,
-                       minimal_digraph,
-                       update_surface_units)
+from .digraph import (digraph_from_units,
+                      digraph_from_units_and_streams, 
+                      finalize_digraph,
+                      minimal_digraph,
+                      update_surface_units)
 from thermosteam import Stream
 from ._unit import Unit
 from ._facility import Facility
@@ -22,7 +22,7 @@ from ._system import System
 from ._network import Network
 
 try:
-	from ._digraph.widget import FlowsheetWidget
+	from .digraph.widget import FlowsheetWidget
 except:
 	pass
 
@@ -62,7 +62,7 @@ class Flowsheets:
             super().__delattr__(key)
     
     def __repr__(self):
-        return f'Register:\n ' + '\n '.join([repr(i) for i in self])
+        return 'Register:\n ' + '\n '.join([repr(i) for i in self])
     
     
 class Flowsheet:
@@ -70,7 +70,8 @@ class Flowsheet:
     Create a Flowsheet object which stores references to all stream, unit,
     and system objects. For a tutorial on flowsheets, visit
     :doc:`tutorial/Managing_flowsheets`.
-	"""
+	
+    """
     line = "Flowsheet"
     
     #: [Register] All flowsheets.
@@ -167,7 +168,7 @@ class Flowsheet:
         elif kind == 'minimal':
             f = minimal_digraph(self.ID, self.units, self.streams, **graph_attrs)
         else:
-            raise ValueError(f"kind must be either 'thorough', 'surface', or 'minimal'.")
+            raise ValueError("kind must be either 'thorough', 'surface', or 'minimal'.")
         finalize_digraph(f, file, format)
     
     def _surface_digraph(self, format, **graph_attrs):
@@ -200,7 +201,7 @@ class Flowsheet:
             End streams of the system which are not products. Specify this argument
 			if only a section of the system is wanted, or if recycle streams should be 
 			ignored.
-        facility_recycle : [:class:`~thermosteam.Stream`], optional
+        facility_recycle : :class:`~thermosteam.Stream`, optional
             Recycle stream between facilities and system path. This argument
             defaults to the outlet of a BlowdownMixer facility (if any).
         
@@ -225,6 +226,8 @@ class Flowsheet:
         
         Parameters
         ----------
+        feeds : Iterable[:class:`~thermosteam.Stream`]
+            Feeds to the process.
         ends : Iterable[:class:`~thermosteam.Stream`]
             End streams of the system which are not products. Specify this argument
 			if only a section of the system is wanted, or if recycle streams should be 
