@@ -7,6 +7,8 @@
 # for license details.
 """
 """
+from warnings import warn
+from biosteam.exceptions import GraphicsWarning
 from .utils import colors
 
 __all__ = ('UnitGraphics',
@@ -42,13 +44,49 @@ class UnitGraphics:
         # [function(node, unit)] tailor node to unit.
         self.tailor_node_to_unit = tailor_node_to_unit
     
+    def get_inlet_options(self, sink, sink_index):
+        edge_in = self.edge_in
+        try:
+            options = edge_in[sink_index]
+        except IndexError:
+            if sink._ins_size_is_fixed: 
+                N_inlets = len(edge_in)
+                warn(f'inlet #{sink_index} at {repr(sink)} missing graphics options; '
+                     f'expected at most {N_inlets} inlet' + ('' if N_inlets == 1 else 's'),
+                      GraphicsWarning)
+            options = {'headport': 'c'}
+        return options
+    
+    def get_outlet_options(self, source, source_index):
+        edge_out = self.edge_out
+        try:
+            options = edge_out[source_index]
+        except IndexError:
+            if source._outs_size_is_fixed: 
+                N_outlets = len(edge_out)
+                warn(f'outlet #{source_index} at {repr(source)} missing graphics options; '
+                     f'expected at most {N_outlets} outlet' + ('' if N_outlets == 1 else 's'),
+                      GraphicsWarning)
+            options = {'tailport': 'c'}
+        return options
+    
     @classmethod
     def box(cls, N_ins, N_outs):
         edge_in = [{'headport': 'c'} for i in range(N_ins)]
         edge_out = [{'tailport': 'c'} for i in range(N_outs)]
         return cls(edge_in, edge_out, box_node)
     
-    def get_node_tailored_to_unit(self, unit):
+    def get_minimal_node(self, unit):
+        """Return minmal node (a single dot)."""
+        return dict(
+            name = unit.ID,
+            width = '0.1',
+            shape = 'oval',
+            fillcolor = "#d2d2d2:white",
+            style = 'filled',
+        )
+    
+    def get_node_tailored_to_unit(self, unit): # pragma: no coverage
         """Return node tailored to unit specifications"""
         node = self.node
         node['name'] = unit.ID + '\n' + unit.line
@@ -57,7 +95,7 @@ class UnitGraphics:
             tailor_node_to_unit(node, unit)
         return node
         
-    def __repr__(self):
+    def __repr__(self): # pragma: no coverage
         return f'{type(self).__name__}(node={self.node}, edge_in={self.edge_in}, edge_out={self.edge_out})'
 
 
@@ -91,7 +129,7 @@ box_graphics = UnitGraphics(single_edge_in, single_edge_out, box_node)
 node = box_node.copy()
 node['shape'] = 'triangle'
 node['orientation'] = '270'
-mixer_graphics = UnitGraphics(6 * single_edge_in, right_edge_out, node)
+mixer_graphics = UnitGraphics(multi_edge_in, right_edge_out, node)
 
 # Create splitter graphics
 node = box_node.copy()
@@ -107,16 +145,21 @@ node['height'] = '1.2'
 vertical_column_graphics = UnitGraphics(single_edge_in, top_bottom_edge_out, node)
 
 # Create flash column graphics
-node = node.copy()
+node = box_node.copy()
 node['height'] = '1.1'
 vertical_vessel_graphics = UnitGraphics(single_edge_in, top_bottom_edge_out, node)
 
+# Mixer-Settler graphics
+node = box_node.copy()
+node['width'] = '1.2'
+mixer_settler_graphics = UnitGraphics(multi_edge_in, top_bottom_edge_out, node)
+
 # Single stream heat exchanger node
 node = box_node.copy()
 node['shape'] = 'circle'
 node['color'] = 'none'
 node['margin'] = '0'
-def tailor_utility_heat_exchanger_node(node, unit):
+def tailor_utility_heat_exchanger_node(node, unit): # pragma: no coverage
     try:
         si = unit.ins[0]
         so = unit.outs[0]
@@ -139,7 +182,7 @@ def tailor_utility_heat_exchanger_node(node, unit):
     node['name'] = unit.ID + "\n" + line
 
 utility_heat_exchanger_graphics = UnitGraphics(single_edge_in, single_edge_out, node,
-                                           tailor_utility_heat_exchanger_node)
+                                               tailor_utility_heat_exchanger_node)
 
 # Process heat exchanger network
 node = node.copy()
@@ -148,11 +191,11 @@ node['color'] = 'none'
 node['margin'] = '0'
 node['gradientangle'] = '90'
 node['fillcolor'] = '#cfecf0:#fad6d8'
-def tailor_process_heat_exchanger_node(node, unit):
+def tailor_process_heat_exchanger_node(node, unit): # pragma: no coverage
     node['name'] = unit.ID + "\n Heat exchanger"
 
 process_heat_exchanger_graphics = UnitGraphics(2 * single_edge_in, 2 *single_edge_out, node,
-                                           tailor_process_heat_exchanger_node)
+                                               tailor_process_heat_exchanger_node)
 
 # Process specification graphics
 orange = colors.orange_tint.tint(50)
@@ -161,12 +204,12 @@ node = box_node.copy()
 node['fillcolor'] = orange_tint.HEX + ':' + orange.HEX
 node['shape'] = 'note'
 node['margin'] = '0.2'
-def tailor_process_specification_node(node, unit):
+def tailor_process_specification_node(node, unit): # pragma: no coverage
     node['name'] = (f"{unit.ID} - {unit.description}\n"
                     f"{unit.line}")
 
 process_specification_graphics = UnitGraphics(single_edge_in, single_edge_out, node,
-                                          tailor_process_specification_node)
+                                              tailor_process_specification_node)
 
 # System unit for creating diagrams
 node = box_node.copy()
@@ -179,7 +222,7 @@ stream_unit = UnitGraphics(multi_edge_in, multi_edge_out, node)
 
 
 node = box_node.copy()
-def tailor_junction_node(node, unit):
+def tailor_junction_node(node, unit): # pragma: no coverage
     if not any(unit._get_streams()):
         node['fontsize'] = '18'
         node['shape'] = 'plaintext'
@@ -190,4 +233,4 @@ def tailor_junction_node(node, unit):
         node['color'] = node['fillcolor'] = 'black'
 
 junction_graphics = UnitGraphics(single_edge_in, single_edge_out, node,
-                             tailor_junction_node)
\ No newline at end of file
+                                 tailor_junction_node)
\ No newline at end of file
