@@ -61,7 +61,7 @@ class UtilityAgent(Stream):
                  '_regeneration_price', 'heat_transfer_efficiency')
     def __init__(self, ID='', flow=(), phase='l', T=298.15, P=101325., units='kmol/hr',
                  thermo=None, T_limit=None, heat_transfer_price=0.0,
-                 regeneration_price=0.0, heat_transfer_efficiency=1.0,
+                 regeneration_price=0.0, heat_transfer_efficiency=1.0, 
                  **chemical_flows):
         self._thermal_condition = ThermalCondition(T, P)
         thermo = self._load_thermo(thermo)
@@ -70,7 +70,7 @@ class UtilityAgent(Stream):
             name, factor = self._get_flow_name_and_factor(units)
             flow = getattr(self, name)
             flow[:] = self.mol / factor
-        self._sink = self._source = None
+        self._link = self._sink = self._source = None
         self._init_cache()
         self._register(ID)
         self.T_limit = T_limit
@@ -78,6 +78,20 @@ class UtilityAgent(Stream):
         self.regeneration_price = regeneration_price
         self.heat_transfer_efficiency = heat_transfer_efficiency
     
+    @property
+    def price(self):
+        raise AttributeError(f"'{type(self).__name__}' object has no attribute 'price'")
+    @price.setter
+    def price(self, price):
+        raise AttributeError(f"'{type(self).__name__}' object has no attribute 'price'")
+    
+    @property
+    def cost(self):
+        raise AttributeError(f"'{type(self).__name__}' object has no attribute 'cost'")
+    @cost.setter
+    def cost(self, cost):
+        raise AttributeError(f"'{type(self).__name__}' object has no attribute 'cost'")
+    
     def to_stream(self, ID=None):
         """
         Return a copy as a :class:`~thermosteam.Stream` object.
@@ -95,22 +109,15 @@ class UtilityAgent(Stream):
         
         """
         new = Stream.__new__(Stream)
-        new._sink = new._source = None
+        new._link = new._sink = new._source = None
         new._thermo = self._thermo
         new._imol = self._imol.copy()
         new._thermal_condition = self._thermal_condition.copy()
         new._init_cache()
-        new.price = 0
+        new._price = 0.
         new.ID = ID
         return new
     
-    @property
-    def cost(self):
-        return 0
-    @property
-    def price(self):
-        return 0
-    
     @property
     def heat_transfer_price(self):
         """Price of transfered heat [USD/kJ]."""
@@ -149,41 +156,6 @@ class UtilityAgent(Stream):
     def __repr__(self):
         return f"<{type(self).__name__}: {self.ID}>"
 
-
-# Create cooling agnets
-thermo_water = Thermo(['Water'])
-cooling_water = UtilityAgent('cooling_water',
-                             Water=1, T=305.372, P=101325,
-                             thermo=thermo_water,
-                             T_limit = 324.817,
-                             regeneration_price = 4.8785e-4)
-chilled_water = UtilityAgent('chilled_water',
-                             Water=1, T=280.372, P=101325,
-                             thermo=thermo_water,
-                             T_limit = 300.372,
-                             heat_transfer_price = 5e-6)
-chilled_brine = UtilityAgent('chilled_brine',
-                             Water=1, T=255.372, P=101325,
-                             thermo=thermo_water,
-                             T_limit = 275.372,
-                             heat_transfer_price = 8.145e-6,)
-
-# Create heating agents
-low_pressure_steam = UtilityAgent('low_pressure_steam',
-                                  Water=1, T=412.189, P=344738.0, phase='g',
-                                  thermo=thermo_water,
-                                  regeneration_price = 0.2378,
-                                  heat_transfer_efficiency = 0.95)
-medium_pressure_steam = UtilityAgent('medium_pressure_steam',
-                                     Water=1, T=454.770, P=1.041e+6, phase='g',
-                                     thermo=thermo_water,
-                                     regeneration_price = 0.2756,
-                                     heat_transfer_efficiency = 0.90) 
-high_pressure_steam = UtilityAgent('high_pressure_steam',
-                                   Water=1, T=508.991, P=3.11e+6, phase='g', 
-                                   thermo=thermo_water,
-                                   regeneration_price = 0.3171,
-                                   heat_transfer_efficiency = 0.85) 
        
 # %%
 
@@ -269,8 +241,8 @@ class HeatUtility:
     #: [DisplayUnits] Units of measure for IPython display
     display_units = DisplayUnits(duty='kJ/hr', flow='kmol/hr', cost='USD/hr')
 
-    cooling_agents = [cooling_water, chilled_water, chilled_brine]
-    heating_agents = [low_pressure_steam, medium_pressure_steam, high_pressure_steam]
+    #: [Thermo] Used broadly throughout BioSTEAM utilities
+    thermo_water = Thermo(['Water'])
 
     def __init__(self, heat_transfer_efficiency=None, heat_exchanger=None):
         self.heat_transfer_efficiency = heat_transfer_efficiency
@@ -290,6 +262,70 @@ class HeatUtility:
         """[str] ID of utility agent being used."""
         self.agent = self.get_agent(ID)
 
+    @classmethod
+    def default_agents(cls):
+        """Reset all agents back to BioSTEAM's defaults."""
+        cls.default_heating_agents()
+        cls.default_cooling_agents()
+
+    @classmethod
+    def default_heating_agents(cls):
+        """Reset all heating agents back to BioSTEAM's defaults."""
+        thermo_water = cls.thermo_water
+        low_pressure_steam = UtilityAgent(
+            'low_pressure_steam',
+            Water=1, T=412.189, P=344738.0, phase='g',
+            thermo=thermo_water,
+            regeneration_price = 0.2378,
+            heat_transfer_efficiency = 0.95,
+        )
+        medium_pressure_steam = UtilityAgent(
+            'medium_pressure_steam',
+            Water=1, T=454.770, P=1.041e+6, phase='g',
+            thermo=thermo_water,
+            regeneration_price = 0.2756,
+            heat_transfer_efficiency = 0.90,
+        ) 
+        high_pressure_steam = UtilityAgent(
+            'high_pressure_steam',
+            Water=1, T=508.991, P=3.11e+6, phase='g', 
+            thermo=thermo_water,
+            regeneration_price = 0.3171,
+            heat_transfer_efficiency = 0.85,
+        ) 
+        cls.heating_agents = [low_pressure_steam, 
+                              medium_pressure_steam, 
+                              high_pressure_steam]
+        
+    @classmethod
+    def default_cooling_agents(cls):
+        """Reset all cooling agents back to BioSTEAM's defaults."""
+        thermo_water = cls.thermo_water
+        cooling_water = UtilityAgent(
+            'cooling_water',
+            Water=1, T=305.372, P=101325,
+            thermo=thermo_water,
+            T_limit = 324.817,
+            regeneration_price = 4.8785e-4,
+        )
+        chilled_water = UtilityAgent(
+            'chilled_water',
+            Water=1, T=280.372, P=101325,
+            thermo=thermo_water,
+            T_limit = 300.372,
+            heat_transfer_price = 5e-6
+        )
+        chilled_brine = UtilityAgent(
+            'chilled_brine',
+            Water=1, T=255.372, P=101325,
+            thermo=thermo_water,
+            T_limit = 275.372,
+            heat_transfer_price = 8.145e-6,
+        )
+        cls.cooling_agents = [cooling_water, 
+                              chilled_water, 
+                              chilled_brine]
+
     def copy_like(self, other):
         """Copy all data from another heat utility."""
         self.inlet_utility_stream = other.inlet_utility_stream.copy()
@@ -403,14 +439,10 @@ class HeatUtility:
     @staticmethod
     def heat_utilities_by_agent(heat_utilities):
         """Return a dictionary of heat utilities sorted by agent."""
-        heat_utilities_by_agent = {}
-        for heat_utility in heat_utilities:
-            agent = heat_utility.agent
-            if not agent: continue
-            if agent in heat_utilities_by_agent:
-                heat_utilities_by_agent[agent].append(heat_utility)
-            else:
-                heat_utilities_by_agent[agent] = [heat_utility]
+        heat_utilities = [i for i in heat_utilities if i.agent]
+        heat_utilities_by_agent = {i.agent: [] for i in heat_utilities}
+        for i in heat_utilities:
+            heat_utilities_by_agent[i.agent].append(i)
         return heat_utilities_by_agent
 
     @classmethod
@@ -576,9 +608,9 @@ class HeatUtility:
         """Return string related to specifications"""
         if not self.agent:
             return (f'{type(self).__name__}: None\n'
-                    +f' duty: 0\n'
-                    +f' flow: 0\n'
-                    +f' cost: 0')
+                    +' duty: 0\n'
+                    +' flow: 0\n'
+                    +' cost: 0')
         else:
             (duty, flow, cost, duty_units,
              flow_units, cost_units) = self._info_data(duty, flow, cost)
@@ -593,3 +625,4 @@ class HeatUtility:
         print(self._info(duty, flow, cost))
     _ipython_display_ = show
 
+HeatUtility.default_agents()
\ No newline at end of file
