@@ -11,6 +11,8 @@ from thermosteam.units_of_measure import DisplayUnits, convert
 
 __all__ = ('PowerUtility',)
 
+default_price = 0.0782
+
 class PowerUtility:
     """
     Create an PowerUtility object that, when called, calculates the cost of 
@@ -20,46 +22,44 @@ class PowerUtility:
     --------
     Create a PowerUtility object:
     
-    .. code-block:: python
-    
-       >>> pu = PowerUtility()
-       >>> pu
-       <PowerUtility: 0 kW, 0 USD/hr>
+    >>> pu = PowerUtility()
+    >>> pu
+    <PowerUtility: 0 kW, 0 USD/hr>
        
     Call object to calculate cost:
         
-    .. code-block:: python
-    
-       >>> pu(rate=500.)
-       >>> pu
-       <PowerUtility: 500 kW, 39.1 USD/hr>
+    >>> pu(rate=500.)
+    >>> pu
+    <PowerUtility: 500 kW, 39.1 USD/hr>
        
     Results are accessible:
         
-    .. code-block:: python
+    >>> pu.rate, pu.cost
+    (500.0, 39.1)
     
-       >>> pu.rate, pu.cost
-       (500.0, 39.1)
-    
-    Notes
-    -----
     PowerUtility objects have `consumption` and `production` attributes
     which are updated when setting the rate with the assumption that
     a positive rate means no production (only consumption) and a negative
     rate means no consumption (only consumption).
     
+    >>> pu.consumption, pu.production
+    (500.0, 0.0)
+    
     It is possible to have both consumption and production by setting these
     attributes individually (instead of setting rate)
     
+    >>> pu.production = 100. 
+    >>> pu.rate
+    400.0
+    
+    Notice how the rate is equal the consumption minus the production.
+    
     """
     __slots__ = ('consumption', 'production')
     
     #: [DisplayUnits] Units of measure for IPython display
     display_units = DisplayUnits(rate='kW', cost='USD/hr')
     
-    #: [float] USD/kWhr
-    price = 0.0782
-    
     def __init__(self):
         #: Electricity consumption [kW]
         self.consumption = 0.
@@ -67,6 +67,11 @@ class PowerUtility:
         #: Electricity production [kW]
         self.production = 0.
     
+    @classmethod
+    def default_price(cls):
+        """Reset price back to BioSTEAM's default."""
+        cls.price = default_price #: [float] USD/kWhr
+    
     @property
     def rate(self):
         """Power requirement [kW]."""
@@ -79,7 +84,7 @@ class PowerUtility:
             self.production = 0.
         else:
             self.consumption = 0.
-            self.production = rate
+            self.production = -rate
     
     @property
     def cost(self):
@@ -87,7 +92,7 @@ class PowerUtility:
         return self.price * self.rate
     
     def __bool__(self):
-        return bool(self.rate)
+        return bool(self.consumption or self.production)
     
     def __call__(self, rate):
         """Set rate in kW."""
@@ -98,6 +103,16 @@ class PowerUtility:
         self.consumption = sum([i.consumption for i in power_utilities])
         self.production = sum([i.production for i in power_utilities])
     
+    def copy_like(self, power_utility):
+        """Copy consumption anf production rates from another power utility."""
+        self.consumption = power_utility.consumption
+        self.production = power_utility.production
+    
+    def scale(self, scale):
+        """Scale consumption and production accordingly."""
+        self.consumption *= scale
+        self.production *= scale
+    
     @classmethod
     def sum(cls, power_utilities):
         """
@@ -118,4 +133,6 @@ class PowerUtility:
     _ipython_display = show    
     
     def __repr__(self):
-        return (f'<{type(self).__name__}: {self.rate:.3g} kW, {self.cost:.3g} USD/hr>')    
\ No newline at end of file
+        return (f'<{type(self).__name__}: {self.rate:.3g} kW, {self.cost:.3g} USD/hr>')
+    
+PowerUtility.default_price()
\ No newline at end of file
