@@ -8,44 +8,28 @@
 """
 """
 import flexsolve as flx
-from ._digraph import (digraph_from_units_and_streams,
-                       minimal_digraph,
-                       surface_digraph,
-                       finalize_digraph)
-from thermosteam import Stream
+from .digraph import (digraph_from_units_and_streams,
+                      digraph_from_system,
+                      minimal_digraph,
+                      surface_digraph,
+                      finalize_digraph)
+from thermosteam import Stream, MultiStream
 from thermosteam.utils import registered
 from .exceptions import try_method_with_object_stamp
 from ._network import Network
 from ._facility import Facility
 from ._unit import Unit
-from ._report import save_report
+from .report import save_report
 from .exceptions import InfeasibleRegion
 from .utils import colors, strtuple
+from collections.abc import Iterable
+from collections import deque
+from warnings import warn
 import biosteam as bst
+import numpy as np
 
 __all__ = ('System',)    
 
-# %% Functions for building systems
-
-def streams_from_path(path):
-    isa = isinstance
-    streams = set()
-    for i in path:
-        if isa(i, System):
-            streams.add(i.streams)
-        elif isa(i, Unit):
-            streams.update(i._ins + i._outs)
-    return streams
-
-def feeds_from_streams(streams):
-    return {s for s in streams if not s._source}
-
-def products_from_streams(streams):
-    return {s for s in streams if not s._sink}
-
-def filter_out_missing_streams(streams):
-    streams.intersection_update([i for i in streams if i])
-
 # %% Functions for taking care of numerical specifications within a system path
     
 def run_unit_in_path(unit):
@@ -67,10 +51,12 @@ def converge_system_in_path(system):
 def simulate_unit_in_path(unit):
     specification = unit._specification
     if specification:
-        method = specification
+        try_method_with_object_stamp(unit, unit._load_stream_links)
+        try_method_with_object_stamp(unit, unit._setup)
+        try_method_with_object_stamp(unit, specification)
+        try_method_with_object_stamp(unit, unit._summary)
     else:
-        method = unit.simulate
-    try_method_with_object_stamp(unit, method)
+        try_method_with_object_stamp(unit, unit.simulate)
 
 def simulate_system_in_path(system):
     specification = system._specification
@@ -82,6 +68,12 @@ def simulate_system_in_path(system):
 
 # %% Debugging and exception handling
 
+def raise_recycle_type_error(recycle):
+    raise ValueError(
+       f"invalid recycle of type '{type(recycle).__name__}' encountered; "
+        "recycle must be either a Stream object, a tuple of Stream objects, or None"
+    )
+
 def _evaluate(self, command=None):
     """
     Evaluate a command and request user input for next command.
@@ -108,7 +100,7 @@ def _evaluate(self, command=None):
         except Exception as err:
             # Print exception and ask to raise error or continue evaluating
             err = colors.exception(f'{type(err).__name__}:') + f' {str(err)}\n\n'
-            info = colors.info(f"Enter to raise error or type to evaluate:\n")
+            info = colors.info("Enter to raise error or type to evaluate:\n")
             command = input(err + info + ">>> ")
             if command == '': raise err
             _evaluate(self, command)        
@@ -137,7 +129,7 @@ def _method_debug(self, func):
 def _notify_run_wrapper(self, func):
     """Decorate a System run method to notify you after each loop"""
     def wrapper(*args, **kwargs):
-        if self.recycle:
+        if self._recycle:
             func(*args, **kwargs)
             input(f'        Finished loop #{self._iter}\n')
         else:
@@ -150,25 +142,8 @@ def _notify_run_wrapper(self, func):
 
 # %% Process flow
 
-class system(type):
-    @property
-    def converge_method(self):
-        """Iterative convergence method ('wegstein', 'aitken', or 'fixed point')."""
-        return self._converge_method.__name__[1:]
-    @converge_method.setter
-    def converge_method(self, method):
-        method = method.lower().replace('-', '').replace(' ', '')
-        if 'wegstein' == method:
-            self._converge_method = self._wegstein
-        elif 'fixedpoint' == method:
-            self._converge_method = self._fixed_point
-        elif 'aitken' == method:
-            self._converge_method = self._aitken
-        else:
-            raise ValueError(f"only 'wegstein', 'aitken', and 'fixed point' methods are valid, not '{method}'")
-
 @registered('SYS')
-class System(metaclass=system):
+class System:
     """
     Create a System object that can iteratively run each element in a path
     of BioSTREAM objects until the recycle stream is converged. A path can
@@ -182,26 +157,56 @@ class System(metaclass=system):
          registered in flowsheet.
     path : tuple[Unit, function and/or System]
         A path that is run element by element until the recycle converges.
-    recycle=None : :class:`~thermosteam.Stream`, optional
+    recycle=None : :class:`~thermosteam.Stream` or tuple[:class:`~thermosteam.Stream`], optional
         A tear stream for the recycle loop.
     facilities=() : tuple[Unit, function, and/or System], optional
         Offsite facilities that are simulated only after
         completing the path simulation.
+    facility_recycle : :class:`~thermosteam.Stream`, optional
+        Recycle stream between facilities and system path.
+    N_runs : int, optional
+        Number of iterations to run system. This parameter is applicable 
+        only to systems with no recycle loop.
 
     """
+    __slots__ = (
+        '_ID',
+        '_path',
+        '_facilities',
+        '_facility_loop',
+        '_recycle',
+        '_N_runs',
+        '_specification',
+        '_mol_error',
+        '_T_error',
+        '_iter',
+        'units',
+        'streams',
+        'subsystems',
+        'feeds',
+        'products',
+        'maxiter',
+        'molar_tolerance',
+        'temperature_tolerance',
+        'flowsheet',
+        '_converge_method',
+        '_costunits',
+        '_TEA',
+    )
+    
     ### Class attributes ###
     
-    #: Maximum number of iterations
-    maxiter = 100
+    #: [int] Default maximum number of iterations
+    default_maxiter = 200
     
-    #: Molar tolerance (kmol/hr)
-    molar_tolerance = 0.50
+    #: [float] Default molar tolerance (kmol/hr)
+    default_molar_tolerance = 0.50
     
-    #: Temperature tolerance (K)
-    temperature_tolerance = 0.10
+    #: [float] Default temperature tolerance (K)
+    default_temperature_tolerance = 0.10
 
-    # [dict] Cached downstream systems by (system, unit, with_facilities) keys
-    _cached_downstream_systems = {} 
+    #: [str] Default convergence method.
+    default_converge_method = 'Aitken'
 
     @classmethod
     def from_feedstock(cls, ID, feedstock, feeds=None, facilities=(), 
@@ -250,90 +255,148 @@ class System(metaclass=system):
         """
         facilities = Facility.ordered_facilities(facilities)
         isa = isinstance
-        path = tuple([(cls.from_network('', i) if isa(i, Network) else i)
-                      for i in network.path])
+        path = [(cls.from_network('', i) if isa(i, Network) else i)
+                for i in network.path]
         self = cls.__new__(cls)
         self.units = network.units
         self.streams = streams = network.streams
         self.feeds = feeds = network.feeds
         self.products = products = network.products
-        self._specification = None
-        self._set_recycle(network.recycle)
-        self._reset_errors()
+        self.recycle = network.recycle
         self._set_path(path)
+        self._specification = None
+        self._load_flowsheet()
+        self._reset_errors()
         self._set_facilities(facilities)
         self._set_facility_recycle(facility_recycle)
         self._register(ID)
         if facilities:
-            f_streams = streams_from_path(facilities)
-            f_feeds = feeds_from_streams(f_streams)
-            f_products = products_from_streams(f_streams)
+            f_streams = bst.utils.streams_from_path(facilities)
+            f_feeds = bst.utils.feeds(f_streams)
+            f_products = bst.utils.products(f_streams)
             streams.update(f_streams)
             feeds.update(f_feeds)
             products.update(f_products)
         self._finalize_streams()
+        self._load_defaults()
         return self
         
-    def __init__(self, ID, path, recycle=None, facilities=(), facility_recycle=None):
+    def __init__(self, ID, path, recycle=None, facilities=(), 
+                 facility_recycle=None, N_runs=None):
+        self.recycle = recycle
+        self.N_runs = N_runs
+        self._set_path(path)
         self._specification = None
-        self._set_recycle(recycle)
         self._load_flowsheet()
         self._reset_errors()
-        self._set_path(path)
-        self._load_units()
         self._set_facilities(facilities)
         self._set_facility_recycle(facility_recycle)
         self._load_streams()
         self._finalize_streams()
         self._register(ID)
+        self._load_defaults()
+    
+    def __eq__(self, other):
+        return (isinstance(other, System) 
+                and self._path == other._path 
+                and self._facilities == other._facilities 
+                and self._recycle == other._recycle)
+    
+    def _load_defaults(self):
+        #: [int] Maximum number of iterations.
+        self.maxiter = self.default_maxiter
+        
+        #: [float] Molar tolerance (kmol/hr)
+        self.molar_tolerance = self.default_molar_tolerance
+        
+        #: [float] Temperature tolerance (K)
+        self.temperature_tolerance = self.default_temperature_tolerance
+        
+        #: [function] Converge method
+        self.converge_method = self.default_converge_method
     
     specification = Unit.specification
     save_report = save_report
     
+    def _extend_flattend_path_and_recycles(self, path, recycles, stacklevel):
+        isa = isinstance
+        recycle = self._recycle
+        stacklevel += 1
+        if recycle:
+            if isa(recycle, Stream):
+                recycles.append(recycle)
+            elif isa(recycle, Iterable):
+                recycles.extend(recycle)
+            else:
+                raise_recycle_type_error(recycle)
+        for i in self._path:
+            if isa(i, System):
+                if i.facilities:
+                    warning = RuntimeWarning('subsystem with facilities could not be flattened')
+                    warn(warning, stacklevel=stacklevel)
+                    path.append(i)
+                elif i.specification:
+                    warning = RuntimeWarning('subsystem with specification could not be flattened')
+                    warn(warning, stacklevel=stacklevel)
+                    path.append(i)
+                else:
+                    i._extend_flattend_path_and_recycles(path, recycles, stacklevel)
+            else:
+                path.append(i)
+    
+    def _get_recycle_streams(self):
+        recycles = []
+        recycle = self._recycle
+        if recycle:
+            if isinstance(recycle, Stream):
+                recycles.append(recycle)
+            elif isinstance(recycle, Iterable):
+                recycles.extend(recycle)
+            else:
+                raise_recycle_type_error(recycle)
+        for i in self.subsystems:
+            recycles.extend(i._get_recycle_streams())
+        return recycles
+    
+    def flatten(self):
+        """Flatten system by removing subsystems."""
+        recycles = []
+        path = []
+        self._extend_flattend_path_and_recycles(path, recycles, stacklevel=2)
+        self._path = tuple(path)
+        self._recycle = tuple(recycles)
+        N_recycles = len(recycles)
+        self.molar_tolerance *= N_recycles
+        self.temperature_tolerance *= N_recycles
+    
     def _load_flowsheet(self):
         self.flowsheet = flowsheet_module.main_flowsheet.get_flowsheet()
     
-    def _set_recycle(self, recycle):
-        assert recycle is None or isinstance(recycle, Stream), (
-             "recycle must be a Stream instance or None, not "
-            f"{type(recycle).__name__}"
-        )
-        self._recycle = recycle
-    
     def _set_path(self, path):
         #: tuple[Unit, function and/or System] A path that is run element
         #: by element until the recycle converges.
-        self.path = path
+        self._path = path = tuple(path)
         
-        #: set[System] All subsystems in the system
-        self.subsystems = subsystems = set()
+        #: set[Unit] All units within the system
+        self.units = units = set()
         
-        #: list[Unit] Network of only unit operations
-        self._unit_path = unit_path = []
+        #: list[System] All subsystems in the system
+        self.subsystems = subsystems = []
         
-        #: set[Unit] All units that have costs.
+        #: set[Unit] All units that have costs (including facilities).
         self._costunits = costunits = set()
         
         isa = isinstance
         for i in path:
-            if i in unit_path: continue
             if isa(i, Unit): 
-                unit_path.append(i)
+                units.add(i)
+                if i._design or i._cost:
+                    costunits.add(i)
             elif isa(i, System):
-                unit_path.extend(i._unit_path)
-                subsystems.add(i)
+                subsystems.append(i)
+                units.update(i.units)
                 costunits.update(i._costunits)
     
-        #: set[Unit] All units in the path that have costs
-        self._path_costunits = path_costunits = {i for i in unit_path
-                                                 if i._design or i._cost}
-        costunits.update(path_costunits)
-        
-        
-    def _load_units(self):
-        #: set[Unit] All units within the system
-        self.units = set(self._unit_path) | self._costunits
-    
     def _set_facilities(self, facilities):
         #: tuple[Unit, function, and/or System] Offsite facilities that are simulated only after completing the path simulation.
         self._facilities = facilities = tuple(facilities)
@@ -349,7 +412,7 @@ class System(metaclass=system):
                 if isa(i, Facility) and not i._system: i._system = self
             elif isa(i, System):
                 units.update(i.units)
-                subsystems.add(i)
+                subsystems.append(i)
                 costunits.update(i._costunits)
     
     def _set_facility_recycle(self, recycle):
@@ -370,17 +433,17 @@ class System(metaclass=system):
             streams.update(sys.streams)
         
         #: set[:class:`~thermosteam.Stream`] All feed streams in the system.
-        self.feeds = feeds_from_streams(streams)
+        self.feeds = bst.utils.feeds(streams)
         
         #: set[:class:`~thermosteam.Stream`] All product streams in the system.
-        self.products = products_from_streams(streams)
+        self.products = bst.utils.products(streams)
         
     def _load_stream_links(self):
-        for u in self._unit_path: u._load_stream_links()
+        for u in self.unit_path: u._load_stream_links()
         
     def _filter_out_missing_streams(self):
         for stream_set in (self.streams, self.feeds, self.products):
-            filter_out_missing_streams(stream_set)
+            bst.utils.filter_out_missing_streams(stream_set)
         
     def _finalize_streams(self):
         self._load_stream_links()
@@ -399,79 +462,96 @@ class System(metaclass=system):
     
     @property
     def recycle(self):
-        """[:class:`~thermosteam.Stream`] A tear stream for the recycle loop"""
+        """
+        :class:`~thermosteam.Stream` or Iterable[:class:`~thermosteam.Stream`]
+        A tear stream for the recycle loop.
+        """
         return self._recycle
+    @recycle.setter
+    def recycle(self, recycle):
+        isa = isinstance
+        self._N_runs = None
+        if recycle is None:
+            self._recycle = recycle
+        elif isa(recycle, Stream):
+            self._recycle = recycle
+        elif isa(recycle, Iterable):
+            recycle = tuple(recycle)
+            for i in recycle:
+                if not isa(i, Stream):
+                    raise ValueError("recycle streams must be Stream objects; "
+                                     f"not {type(i).__name__}")                
+            self._recycle = recycle
+        else:
+            raise_recycle_type_error(recycle)
+
+    @property
+    def N_runs(self):
+        """Number of times to run the path."""
+        return self._N_runs
+    @N_runs.setter
+    def N_runs(self, N_runs):
+        if N_runs: self._recycle = None
+        self._N_runs = N_runs
+
+    @property
+    def path(self):
+        """
+        tuple[Unit, function and/or System] A path that is run element by 
+        element until the recycle(s) converges (if any).
+        
+        """
+        return self._path
 
     @property
     def converge_method(self):
-        """Iterative convergence method ('wegstein', 'aitken', or 'fixed point')."""
+        """Iterative convergence method ('wegstein', 'aitken', or 'fixedpoint')."""
         return self._converge_method.__name__[1:]
     @converge_method.setter
     def converge_method(self, method):
-        if self.recycle is None:
-            raise ValueError(
-                "cannot set converge method when no recyle is specified")
         method = method.lower().replace('-', '').replace(' ', '')
-        if 'wegstein' == method:
-            self._converge_method = self._wegstein
-        elif 'fixedpoint' == method:
-            self._converge_method = self._fixed_point
-        elif 'aitken' == method:
-            self._converge_method = self._aitken
-        else:
-            raise ValueError(
-                f"only 'wegstein', 'aitken', and 'fixed point' methods "
-                f"are valid, not '{method}'")
+        try:
+            self._converge_method = getattr(self, '_' + method)
+        except:
+            raise ValueError("only 'wegstein', 'aitken', and 'fixedpoint' "
+                            f"methods are valid, not '{method}'")
 
     
     def _downstream_path(self, unit):
         """Return a list composed of the `unit` and everything downstream."""
         if unit not in self.units: return []
-        elif self._recycle: return self.path
-        unit_found = False
-        downstream_units = unit._downstream_units
-        path = []
+        elif self._recycle: return self._path
         isa = isinstance
-        for i in self.path:
-            if unit_found:
-                if isa(i, System):
-                    for u in i.units:
-                        if u in downstream_units:
-                            path.append(i)
-                            break
-                elif i in downstream_units or not isa(i, Unit):
-                    path.append(i)
-            else:
-                if unit is i:
-                    unit_found = True
-                    path.append(unit)
-                elif isa(i, System) and unit in i.units:
-                    unit_found = True   
-                    path.append(i)
-        return path
-
+        for index, i in enumerate(self._path):
+            if unit is i:
+                return self._path[index:]
+            elif (isa(i, System) and unit in i.units): 
+                return i._downstream_path(unit) + self._path[index+1:]
+        return []
+    
+    def _downstream_facilities(self, unit):
+        """Return a list of facilities composed of the `unit` and 
+        everything downstream."""
+        isa = isinstance
+        for index, i in enumerate(self._facilities):
+            if unit is i or (isa(i, System) and unit in i.units):
+                return self._facilities[index:]
+        return []
+    
     def _downstream_system(self, unit):
         """Return a system with a path composed of the `unit` and
         everything downstream (facilities included)."""
-        if unit is self.path[0]: return self
-        system = self._cached_downstream_systems.get((self, unit))
-        if system: return system
+        if unit is self._path[0]: return self
         path = self._downstream_path(unit)
         if path:
-            downstream_facilities = self._facilities            
+            facilities = self._facilities            
         else:
-            unit_found = False
-            isa = isinstance
-            for pos, i in enumerate(self._facilities):
-                if unit is i or (isa(i, System) and unit in i.units):
-                    downstream_facilities = self._facilities[pos:]
-                    unit_found = True
-                    break
-            assert unit_found, f'{unit} not found in system'
+            facilities = self._downstream_facilities(unit)
+            if not facilities:
+                raise RuntimeError(f'{unit} not found in system')
         system = System(None, path,
-                        facilities=downstream_facilities)
+                        facilities=facilities)
         system._ID = f'{type(unit).__name__}-{unit} and downstream'
-        self._cached_downstream_systems[unit] = system
         return system
     
     def _minimal_digraph(self, **graph_attrs):
@@ -479,18 +559,23 @@ class System(metaclass=system):
         return minimal_digraph(self.ID, self.units, self.streams, **graph_attrs)
 
     def _surface_digraph(self, **graph_attrs):
-        return surface_digraph(self.path)
+        return surface_digraph(self._path)
 
     def _thorough_digraph(self, **graph_attrs):
-        return digraph_from_units_and_streams(self.units, self.streams, 
+        return digraph_from_units_and_streams(self.unit_path + [i for i in self.facilities if isinstance(i, (Unit, System))], 
+                                              self.streams, 
                                               **graph_attrs)
         
+    def _cluster_digraph(self, **graph_attrs):
+        return digraph_from_system(self, **graph_attrs)
+        
     def diagram(self, kind='surface', file=None, format='png', **graph_attrs):
         """Display a `Graphviz <https://pypi.org/project/graphviz/>`__ diagram of the system.
         
         Parameters
         ----------
-        kind='surface' : {'thorough', 'surface', 'minimal'}:
+        kind='surface' : 'cluster', 'thorough', 'surface', or 'minimal'
+            * **'cluster':** Display all units clustered by system.
             * **'thorough':** Display every unit within the path.
             * **'surface':** Display only elements listed in the path.
             * **'minimal':** Display path as a box.
@@ -500,14 +585,16 @@ class System(metaclass=system):
             File format (e.g. "png", "svg").
         
         """
-        if kind == 'thorough':
+        if kind == 'cluster':
+            f = self._cluster_digraph(format=format, **graph_attrs)
+        elif kind == 'thorough':
             f = self._thorough_digraph(format=format, **graph_attrs)
         elif kind == 'surface':
             f = self._surface_digraph(format=format, **graph_attrs)
         elif kind == 'minimal':
             f = self._minimal_digraph(format=format, **graph_attrs)
         else:
-            raise ValueError(f"kind must be either 'thorough', 'surface', or 'minimal'")
+            raise ValueError("kind must be either 'cluster', 'thorough', 'surface', or 'minimal'")
         finalize_digraph(f, file, format)
             
     # Methods for running one iteration of a loop
@@ -530,13 +617,13 @@ class System(metaclass=system):
         """
         if (mol < 0.).any():
             raise InfeasibleRegion('material flow')
-        recycle = self.recycle
-        rmol = recycle.mol
-        rmol[:] = mol
-        T = recycle.T
+        self._set_recycle_data(mol)
+        T = self._get_recycle_temperatures()
         self._run()
-        self._mol_error = mol_error = abs(mol - recycle.mol).sum()
-        self._T_error = T_error = abs(T - recycle.T)
+        rmol = self._get_recycle_data()
+        rT = self._get_recycle_temperatures()
+        self._mol_error = mol_error = abs(mol - rmol).sum()
+        self._T_error = T_error = np.sum(abs(T - rT))
         self._iter += 1
         if mol_error < self.molar_tolerance and T_error < self.temperature_tolerance:
             unconverged = False
@@ -544,50 +631,134 @@ class System(metaclass=system):
             raise RuntimeError(f'{repr(self)} could not converge' + self._error_info())
         else:
             unconverged = True
-        return rmol.copy(), unconverged
+        return rmol, unconverged
             
+    def _get_recycle_data(self):
+        recycle = self._recycle
+        if isinstance(recycle, Stream):
+            return recycle.imol.data.copy()
+        elif isinstance(recycle, Iterable):
+            return np.vstack([i.imol.data for i in recycle])
+        else:
+            raise RuntimeError('no recycle available')
+    
+    def _set_recycle_data(self, data):
+        recycle = self._recycle
+        isa = isinstance
+        if isa(recycle, Stream):
+            recycle.imol.data[:] = data
+        elif isa(recycle, Iterable):
+            length = len
+            N_rows = data.shape[0]
+            M_rows = sum([length(i) if isa(i, MultiStream) else 1 for i in recycle])
+            if M_rows != N_rows: 
+                raise IndexError(f'expected {M_rows} rows; got {N_rows} rows instead')
+            index = 0
+            for i in recycle:
+                if isa(i, MultiStream):
+                    next_index = index + length(i)
+                    i.imol.data[:] = data[index:next_index, :]
+                    index = next_index
+                else:
+                    i.imol.data[:] = data[index, :]
+                    index += 1
+        else:
+            raise RuntimeError('no recycle available')
+            
+    def _get_recycle_temperatures(self):
+        recycle = self._recycle
+        if isinstance(recycle, Stream):
+            return self._recycle.T
+        elif isinstance(recycle, Iterable):
+            return np.array([i.T for i in recycle], float)
+        else:
+            raise RuntimeError('no recycle available')
+    
     def _setup(self):
         """Setup each element of the system."""
         isa = isinstance
-        for a in self.path:
-            if isa(a, (Unit, System)): a._setup()
-            else: pass # Assume it is a function
+        for i in self._path:
+            if isa(i, (Unit, System)): i._setup()
         
     def _run(self):
-        """Rigorous run each element of the system."""
+        """Run each element in the path. Rotate path if necessary."""
+        if self._recycle:
+            N_elements = len(self._path)
+            if N_elements <= 1:
+                self._run_path()
+            else:
+                max_rotations = N_elements - 1
+                for n in range(N_elements):
+                    try:
+                        self._run_path()
+                    except Exception as error:
+                        if n == max_rotations: raise error
+                        if not n: self._path = deque(self._path)
+                        self._path.rotate()
+                    else:
+                        break
+                if n: self._path = tuple(self._path)
+        else:
+            self._run_path()
+        
+    def _run_path(self):
+        """Rigorously run each element in the path."""
         isa = isinstance
-        for i in self.path:
-            if isa(i, Unit):
+        for i in self._path:
+            if isa(i, Unit):                            
                 run_unit_in_path(i)
             elif isa(i, System): 
                 converge_system_in_path(i)
             else: i() # Assume it is a function
     
+    @property
+    def unit_path(self):
+        """[list] All unit operation as ordered in the path."""
+        unit_path = []
+        isa = isinstance
+        for i in self._path:
+            if isa(i, Unit):
+                unit_path.append(i)
+            elif isa(i, System):
+                unit_path.extend(i.unit_path)
+        return unit_path 
+    
     # Methods for convering the recycle stream    
-    def _fixed_point(self):
+    def _fixedpoint(self):
         """Converge system recycle iteratively using fixed-point iteration."""
-        self._reset_iter()
-        flx.conditional_fixed_point(self._iter_run, self.recycle.mol.copy())
+        self._solve(flx.conditional_fixed_point)
         
     def _wegstein(self):
         """Converge the system recycle iteratively using wegstein's method."""
-        self._reset_iter()
-        flx.conditional_wegstein(self._iter_run, self.recycle.mol.copy())
+        self._solve(flx.conditional_wegstein)
     
     def _aitken(self):
         """Converge the system recycle iteratively using Aitken's method."""
-        self._reset_iter()
-        flx.conditional_aitken(self._iter_run, self.recycle.mol.copy())
-    
-    # Default converge method
-    _converge_method = _aitken
-   
-    def _converge(self):
-        return self._converge_method() if self._recycle else self._run()
+        self._solve(flx.conditional_aitken)
         
-    def _design_and_cost(self):
-        for i in self._path_costunits:
-            try_method_with_object_stamp(i, i._summary)
+    def _solve(self, solver):
+        """Solve the system recycle iteratively using given solver."""
+        self._reset_iter()
+        try:
+            solver(self._iter_run, self._get_recycle_data())
+        except IndexError as error:
+            try:
+                solver(self._iter_run, self._get_recycle_data())
+            except:
+                raise error
+    
+    def _converge(self):
+        if self._N_runs:
+            for i in range(self.N_runs): self._run()
+        elif self._recycle:
+            self._converge_method()
+        else:
+            self._run()
+        
+    def _summary(self):
+        iscallable = callable
+        for i in self._path:
+            if not iscallable(i): try_method_with_object_stamp(i, i._summary)
         isa = isinstance
         for i in self._facilities:
             if isa(i, Unit):
@@ -607,7 +778,7 @@ class System(metaclass=system):
         Stream._default_ID = stream_format if stream_format else ['d', 0]
         streams = set()
         units = set()
-        for i in self._unit_path:
+        for i in self.unit_path:
             if i in units: continue
             try: i.ID = ''
             except: continue
@@ -630,7 +801,7 @@ class System(metaclass=system):
     
     def reset_flows(self):
         """Reset all process streams to zero flow."""
-        from warning import warn
+        from warnings import warn
         warn(DeprecationWarning("'reset_flows' will be depracated; please use 'empty_process_streams'"))
         self.empty_process_streams()
         
@@ -642,23 +813,35 @@ class System(metaclass=system):
             if stream not in feeds: stream.empty()
 
     def empty_recycles(self):
+        """Reset all recycle streams to zero flow."""
         self._reset_errors()        
-        if self.recycle: self.recycle.empty()
+        recycle = self._recycle
+        if recycle:
+            if isinstance(recycle, Stream):
+                recycle.empty()
+            elif isinstance(recycle, Iterable):
+                for i in recycle: i.empty()
+            else:
+                raise_recycle_type_error(recycle)
         for system in self.subsystems:
             system.empty_recycles()
 
+    def reset_cache(self):
+        """Reset cache of all unit operations."""
+        for unit in self.units: unit.reset_cache()
+
     def simulate(self):
         """Converge the path and simulate all units."""
         self._setup()
         self._converge()
-        self._design_and_cost()
+        self._summary()
         if self._facility_loop: self._facility_loop()
         
     # Debugging
     def _debug_on(self):
         """Turn on debug mode."""
         self._run = _notify_run_wrapper(self, self._run)
-        self.path = path = list(self.path)
+        path = self._path
         for i, item in enumerate(path):
             if isinstance(item, Unit):
                 item._run = _method_debug(item, item._run)
@@ -670,7 +853,7 @@ class System(metaclass=system):
     def _debug_off(self):
         """Turn off debug mode."""
         self._run = self._run._original
-        path = self.path
+        path = self._path
         for i, item in enumerate(path):
             if isinstance(item, Unit):
                 item._run = item._run._original
@@ -678,7 +861,6 @@ class System(metaclass=system):
                 item._converge = item._converge._original
             elif callable(item):
                 path[i] = item._original
-        self.path = tuple(path)
     
     def debug(self):
         """Converge in debug mode. Just try it!"""
@@ -689,7 +871,7 @@ class System(metaclass=system):
         if end:
             print(f'\nFinished debugging{end}')
         else:
-            print(f'\n        Finished debugging')
+            print('\n        Finished debugging')
 
     # Representation
     def __str__(self):
@@ -707,24 +889,75 @@ class System(metaclass=system):
     def to_network(self):
         """Return network that defines the system path."""
         isa = isinstance
-        path = [(i.to_network() if isa(i, System) else i) for i in self.path]
+        path = [(i.to_network() if isa(i, System) else i) for i in self._path]
         network = Network.__new__(Network)    
         network.path = path
-        network.recycle = self.recycle
+        network.recycle = self._recycle
         network.units = self.units
         network.subnetworks = [i for i in path if isa(i, Network)]
         network.feeds = self.feeds
         network.products = self.products
         return network
     
+    def print(self, spaces=''): # pragma: no cover
+        """
+        Print in a format that you can use recreate the system.
+        """
+        print(self._stacked_info())
+    
+    def _stacked_info(self, spaces=''): # pragma: no cover
+        """
+        Return info with inner layers of path and facilities stacked.
+        """
+        info = f"{type(self).__name__}({repr(self.ID)}"
+        spaces += 4 * " "
+        dlim = ',\n' + spaces
+        update_info = lambda new_info: dlim.join([info, new_info])
+        def get_path_info(path):
+            isa = isinstance
+            path_info = []
+            for i in path:
+                if isa(i, Unit):
+                    path_info.append(str(i))
+                elif isa(i, System):
+                    path_info.append(i._stacked_info(spaces))
+                else:
+                    path_info.append(str(i))
+            return '[' + (dlim + " ").join(path_info) + ']'
+        path_info = get_path_info(self._path)
+        info = update_info(path_info)
+        facilities = self.facilities
+        if facilities:
+            facilities_info = get_path_info(self.facilities)
+            facilities_info = f'facilities={facilities_info}'
+            info = update_info(facilities_info)
+        recycle = self._recycle
+        if recycle:
+            recycle = self._get_recycle_info()
+            info = update_info(f"recycle={recycle}")
+        if self.N_runs:
+            info = update_info(f"N_runs={self.N_runs}")
+        info += ')'
+        return info
+    
+    def _get_recycle_info(self):
+        recycle = self._recycle
+        if isinstance(recycle, Stream):
+            recycle = recycle._source_info()
+        else:
+            recycle = ", ".join([i._source_info() for i in recycle])
+            recycle = '[' + recycle + ']'
+        return recycle
+    
     def _ipython_display_(self):
-        try: self.diagram('minimal')
-        except: pass
+        if bst.ALWAYS_DISPLAY_DIAGRAMS:
+            try: self.diagram('minimal')
+            except: pass
         self.show()
 
     def _error_info(self):
         """Return information on convergence."""
-        if self.recycle:
+        if self._recycle:
             return (f"\n convergence error: Flow rate   {self._mol_error:.2e} kmol/hr"
                     f"\n                    Temperature {self._T_error:.2e} K"
                     f"\n iterations: {self._iter}")
@@ -733,12 +966,12 @@ class System(metaclass=system):
 
     def _info(self):
         """Return string with all specifications."""
-        if self.recycle is None:
+        if self._recycle is None:
             recycle = ''
         else:
-            recycle = f"\n recycle: {self.recycle}"
+            recycle = f"\n recycle: {self._get_recycle_info()}"
         error = self._error_info()
-        path = strtuple(self.path)
+        path = strtuple(self._path)
         i = 1; last_i = 0
         while True:
             i += 2
@@ -772,32 +1005,35 @@ class System(metaclass=system):
                 + error)
 
 
-class FacilityLoop(metaclass=system):
-    __slots__ = ('system', 'recycle',
-                 '_mol_error', '_T_error', '_iter')
+class FacilityLoop:
+    __slots__ = ('system', '_recycle',
+                 'maxiter', 'molar_tolerance', 'temperature_tolerance',
+                 '_converge_method', '_mol_error', '_T_error', '_iter')
     
-    #: Maximum number of iterations to solve facilities
-    maxiter = 50
-    
-    #: Molar tolerance (kmol/hr)
-    molar_tolerance = 0.50
-    
-    #: Temperature tolerance (K)
-    temperature_tolerance = 0.10
+    default_maxiter = 50
+    default_molar_tolerance = 0.50
+    default_temperature_tolerance = 0.10
+    default_converge_method = System.default_converge_method
     
     def __init__(self, system, recycle):
         self.system = system
-        self.recycle = recycle
+        self._recycle = recycle
         self._reset_errors()
+        self._load_defaults()
         
+    recycle = System.recycle
+    converge_method = System.converge_method
+    _get_recycle_temperatures = System._get_recycle_temperatures
+    _get_recycle_data = System._get_recycle_data
+    _set_recycle_data = System._set_recycle_data
     _reset_errors = System._reset_errors
     _error_info = System._error_info
     _iter_run = System._iter_run
-    _fixed_point = System._fixed_point
+    _fixedpoint = System._fixedpoint
     _aitken = System._aitken
     _wegstein = System._wegstein
-    _converge_method = System._converge_method
-    converge_method = System.converge_method
+    _solve = System._solve
+    _load_defaults = System._load_defaults
     
     def _reset_iter(self):
         self.system._reset_iter()
