@@ -13,10 +13,10 @@ from graphviz import Digraph
 from ._graphics import UnitGraphics, box_graphics
 from thermosteam import Stream
 from ._heat_utility import HeatUtility
-from .utils import Ins, Outs, NotImplementedMethod, \
+from .utils import Inlets, Outlets, NotImplementedMethod, \
                    format_unit_line, static
 from ._power_utility import PowerUtility
-from ._digraph import finalize_digraph
+from .digraph import finalize_digraph
 from thermosteam.utils import thermo_user, registered
 from thermosteam.units_of_measure import convert
 import biosteam as bst
@@ -35,6 +35,8 @@ class Unit:
 
     **Abstract class methods**
     
+    reset_cache()
+        Reset unit operartion cache.
     _setup()
         Set stream conditions and constant data.
     _run()
@@ -84,19 +86,19 @@ class Unit:
         Outlet streams or IDs to initialize output streams.
         If empty, default IDs will be given.
         If None, leave streams missing.
-    thermo=None : Thermo
+    thermo=None : :class:`~thermosteam.Thermo`
         Thermo object to initialize inlet and outlet streams. Defaults to
         `biosteam.settings.get_thermo()`.
     
     Attributes
     ----------
-    ins : Ins[:class:`~thermosteam.Stream`]
+    ins : Inlets[:class:`~thermosteam.Stream`]
         Input streams.
-    outs : Outs[:class:`~thermosteam.Stream`]
+    outs : Outlets[:class:`~thermosteam.Stream`]
         Output streams.
     power_utility : PowerUtility
         Electricity rate requirements are stored here (not including auxiliary units).
-    heat_utilities : tuple[HeatUtility]
+    heat_utilities : tuple[:class:`~biosteam.HeatUtility`]
         Cooling and heating requirements are stored here (not including auxiliary units).
     design_results : dict
         All design requirements (not including auxiliary units).
@@ -105,7 +107,6 @@ class Unit:
     thermo : Thermo
         The thermodynamic property package used by the unit.
     
-    
     Examples
     --------
     :doc:`tutorial/Creating_a_Unit`
@@ -117,8 +118,6 @@ class Unit:
     :doc:`tutorial/Unit_decorators`
     
     """ 
-    # Settings
-    IPYTHON_DISPLAY_UNIT_OPERATIONS = True
     
     def __init_subclass__(cls,
                           isabstract=False,
@@ -132,7 +131,17 @@ class Unit:
         if not isabstract:
             if not hasattr(cls, '_BM'): cls._BM = {}
             if not hasattr(cls, '_units'): cls._units = {}
-            if not cls._run: static(cls)
+            if not cls._run:
+                if cls._N_ins == 1 and cls._N_outs == 1:
+                    static(cls)
+                else:
+                    raise NotImplementedError(
+                        "Unit subclass with multiple inlet or outlet streams "
+                        "must implement a '_run' method unless the "
+                        "'isabstract' keyword argument is True"
+                    )
+        if '__init__' in dct and '_stacklevel' not in dct:
+            cls._stacklevel += 1
         
     ### Abstract Attributes ###
     
@@ -141,11 +150,11 @@ class Unit:
     # without having add these costs in the `purchase_costs` dictionary
     auxiliary_unit_names = ()
     
-    # [int] Expected number of input streams
+    # [int] Expected number of inlet streams
     _N_ins = 1  
     
-    # [int] Expected number of output streams
-    _N_outs = 2  
+    # [int] Expected number of outlet streams
+    _N_outs = 1
     
     # [bool] Whether the number of streams in ins is fixed
     _ins_size_is_fixed = True
@@ -159,8 +168,12 @@ class Unit:
     # [StreamLinkOptions] Options for linking streams
     _stream_link_options = None
     
-    # [biosteam Graphics] a Graphics object for diagram representation.
+    # [biosteam Graphics] A Graphics object for diagram representation
     _graphics = box_graphics
+
+    # [int] Used for piping warnings. Should be equal to 6 plus the number of
+    # wrappers to Unit.__init__
+    _stacklevel = 6
     
     # [str] The general type of unit, regardless of class
     line = 'Unit'
@@ -168,6 +181,7 @@ class Unit:
     ### Other defaults ###
     
     def __init__(self, ID='', ins=None, outs=(), thermo=None):
+        self._register(ID)
         self._specification = None
         self._load_thermo(thermo)
         self._init_ins(ins)
@@ -175,15 +189,14 @@ class Unit:
         self._init_utils()
         self._init_results()
         self._assert_compatible_property_package()
-        self._register(ID)
     
     def _init_ins(self, ins):
-        # Ins[:class:`~thermosteam.Stream`] Input streams
-        self._ins = Ins(self, self._N_ins, ins, self._thermo, self._ins_size_is_fixed)
+        # Inlets[:class:`~thermosteam.Stream`] Input streams
+        self._ins = Inlets(self, self._N_ins, ins, self._thermo, self._ins_size_is_fixed, self._stacklevel)
     
     def _init_outs(self, outs):
-        # Outs[:class:`~thermosteam.Stream`] Output streams
-        self._outs = Outs(self, self._N_outs, outs, self._thermo, self._outs_size_is_fixed)
+        # Outlets[:class:`~thermosteam.Stream`] Output streams
+        self._outs = Outlets(self, self._N_outs, outs, self._thermo, self._outs_size_is_fixed, self._stacklevel)
     
     def _init_utils(self):
         # tuple[HeatUtility] All heat utilities associated to unit
@@ -219,7 +232,10 @@ class Unit:
         """Return unit node attributes for graphviz."""
         try: self._load_stream_links()
         except: pass
-        return self._graphics.get_node_tailored_to_unit(self)
+        if bst.MINIMAL_UNIT_DIAGRAMS:
+            return self._graphics.get_minimal_node(self)
+        else:
+            return self._graphics.get_node_tailored_to_unit(self)
     
     def get_design_result(self, key, units):
         return convert(self.design_results[key], self._units[key], units)
@@ -282,6 +298,7 @@ class Unit:
     __rpow__ = __rsub__
     
     # Abstract methods
+    reset_cache = NotImplementedMethod
     _run = NotImplementedMethod
     _setup = NotImplementedMethod
     _design = NotImplementedMethod
@@ -313,7 +330,7 @@ class Unit:
     def specification(self, specification):
         if specification:
             if not callable(specification):
-                raise ValueError("specification must a callable or None.")
+                raise AttributeError("specification must be callable")
         self._specification = specification
     
     @property
@@ -359,31 +376,38 @@ class Unit:
         self._summary()
 
     def results(self, with_units=True, include_utilities=True,
-                include_total_cost=True):
+                include_total_cost=True, include_installed_cost=False,
+                include_zeros=True, external_utilities=(), key_hook=None):
         """
         Return key results from simulation as a DataFrame if `with_units`
         is True or as a Series otherwise.
         """
         # TODO: Divide this into functions
-        keys = []; addkey = keys.append
+        def addkey(key):
+            if key_hook: key = key_hook(key)
+            keys.append(key)
+        
+        keys = []; 
         vals = []; addval = vals.append
         if with_units:
             if include_utilities:
                 power_utility = self.power_utility
                 if power_utility:
                     addkey(('Power', 'Rate'))
-                    addkey(('Power', 'Cost'))
                     addval(('kW', power_utility.rate))
-                    addval(('USD/hr', power_utility.cost))
-                for heat_utility in HeatUtility.sum_by_agent(self.heat_utilities):
+                    if include_zeros or power_utility.cost:
+                        addkey(('Power', 'Cost'))
+                        addval(('USD/hr', power_utility.cost))
+                for heat_utility in HeatUtility.sum_by_agent(self.heat_utilities + external_utilities):
                     if heat_utility:
                         ID = heat_utility.ID.replace('_', ' ').capitalize()
                         addkey((ID, 'Duty'))
-                        addkey((ID, 'Flow'))
-                        addkey((ID, 'Cost'))
                         addval(('kJ/hr', heat_utility.duty))
+                        addkey((ID, 'Flow'))
                         addval(('kmol/hr', heat_utility.flow))
-                        addval(('USD/hr', heat_utility.cost))
+                        if include_zeros or heat_utility.cost: 
+                            addkey((ID, 'Cost'))
+                            addval(('USD/hr', heat_utility.cost))
             units = self._units
             Cost = self.purchase_costs
             for ki, vi in self.design_results.items():
@@ -401,8 +425,13 @@ class Unit:
             if include_total_cost:
                 addkey(('Total purchase cost', ''))
                 addval(('USD', self.purchase_cost))
-                addkey(('Utility cost', ''))
-                addval(('USD/hr', self.utility_cost))
+                if include_installed_cost:
+                    addkey(('Installed equipment cost', ''))
+                    addval(('USD', self.installed_cost))
+                utility_cost = self.utility_cost
+                if include_zeros or utility_cost: 
+                    addkey(('Utility cost', ''))
+                    addval(('USD/hr', utility_cost))
             if self._GHGs:
                 a, b = self._totalGHG
                 GHG_units =  self._GHG_units
@@ -427,18 +456,19 @@ class Unit:
                 power_utility = self.power_utility
                 if power_utility:
                     addkey(('Power', 'Rate'))
-                    addkey(('Power', 'Cost'))
                     addval(power_utility.rate)
-                    addval(power_utility.cost)
-                for heat_utility in HeatUtility.sum_by_agent(self.heat_utilities):
+                    if include_zeros or power_utility.cost:
+                        addkey(('Power', 'Cost'))
+                        addval(power_utility.cost)
+                for heat_utility in HeatUtility.sum_by_agent(self.heat_utilities + external_utilities):
                     if heat_utility:
-                        if heat_utility:
-                            ID = heat_utility.ID.replace('_', ' ').capitalize()
-                            addkey((ID, 'Duty'))
-                            addkey((ID, 'Flow'))
+                        ID = heat_utility.ID.replace('_', ' ').capitalize()
+                        addkey((ID, 'Duty'))
+                        addval(heat_utility.duty)
+                        addkey((ID, 'Flow'))
+                        addval(heat_utility.flow)
+                        if include_zeros or heat_utility.cost:
                             addkey((ID, 'Cost'))
-                            addval(heat_utility.duty)
-                            addval(heat_utility.flow)
                             addval(heat_utility.cost)
             for ki, vi in self.design_results.items():
                 addkey(('Design', ki))
@@ -467,8 +497,13 @@ class Unit:
             if include_total_cost:
                 addkey(('Total purchase cost', ''))
                 addval(self.purchase_cost)
-                addkey(('Utility cost', ''))
-                addval(self.utility_cost)
+                if include_installed_cost:
+                    addkey(('Installed equipment cost', ''))
+                    addval(self.installed_cost)
+                utility_cost = self.utility_cost
+                if include_zeros or utility_cost:
+                    addkey(('Utility cost', ''))
+                    addval(utility_cost)
             if not keys: return None
             series = pd.Series(vals, pd.MultiIndex.from_tuples(keys))
             series.name = self.ID
@@ -518,8 +553,31 @@ class Unit:
             new_length = len(downstream_units)
         return downstream_units
         
+    def is_upstream_from(self, other):
+        """
+        Return whether this unit is upstream from another unit.
+
+        Parameters
+        ----------
+        other : :class:`~biosteam.Unit`
+            Another unit operation.
+
+        """
+        neighborhood = set()
+        self._add_downstream_neighbors_to_set(neighborhood)
+        direct_neighborhood = neighborhood
+        while True:
+            neighbors = set()
+            for unit in direct_neighborhood:
+                unit._add_downstream_neighbors_to_set(neighbors)
+            if neighbors == direct_neighborhood: break
+            if other in neighbors: return True
+            direct_neighborhood = neighbors
+        return False
+    
     def _neighborhood(self, radius=1, upstream=True, downstream=True):
-        """Return a set of all neighboring units within given radius.
+        """
+        Return a set of all neighboring units within given radius.
         
         Parameters
         ----------
@@ -744,7 +802,7 @@ class Unit:
             info = f'{type(self).__name__}: {self.ID}\n'
         else:
             info = f'{type(self).__name__}\n'
-        info += f'ins...\n'
+        info += 'ins...\n'
         i = 0
         for stream in self.ins:
             if not stream:
@@ -757,7 +815,7 @@ class Unit:
             source_info = f'  from  {type(unit).__name__}-{unit}\n' if unit else '\n'
             info += f'[{i}] {stream.ID}' + source_info + stream_info[index+1:] + '\n'
             i += 1
-        info += f'outs...\n'
+        info += 'outs...\n'
         i = 0
         for stream in self.outs:
             if not stream:
@@ -778,7 +836,7 @@ class Unit:
         print(self._info(T, P, flow, composition, N))
     
     def _ipython_display_(self):
-        if self.IPYTHON_DISPLAY_UNIT_OPERATIONS:
+        if bst.ALWAYS_DISPLAY_DIAGRAMS:
             try: self.diagram()
             except: pass
         self.show()
