@@ -38,7 +38,7 @@ class Block:
     __slots__ = ('_system', '_simulate', '_element')
     
     def __new__(cls, element, system=None):
-        block = cls._blocks.get((system, element))
+        block = cls._blocks.get((id(system), element))
         if block:
             self = block
         else:
@@ -50,22 +50,23 @@ class Block:
         isa = isinstance
         if isa(element, Stream): unit = element.sink
         elif isa(element, Unit): unit = element
+        else: unit = None
         if system and element:
             subsys = system._downstream_system(unit)
             simulate = subsys.simulate
         else:
             subsys = system
-            simulate = unit.simulate if isa(element, Unit) else do_nothing
+            simulate = unit.simulate if unit else do_nothing
         self._system = subsys
         self._simulate = simulate
         self._element = element
-        self._blocks[system, element] = self
+        self._blocks[id(system), element] = self
     
     def parameter(self, setter, simulate, name, distribution, units, 
                   baseline, bounds) -> Parameter:
         """Return a Parameter object."""
         if simulate is None: simulate = self._simulate
-        if not name: name, = signature(setter).parameters.keys()
+        if not name: name, *_ = signature(setter).parameters.keys()
         return Parameter(name, setter, simulate, self._element,
                          self._system, distribution, units, baseline, bounds)
     
