@@ -7,6 +7,11 @@
 # for license details.
 """
 """
-from lazypkg import LazyPkg
+from . import parameter
+from . import in_parallel
 
-LazyPkg(__name__, ['parameter', 'plot', 'in_parallel'])
\ No newline at end of file
+__all__ = (*parameter.__all__,
+           *in_parallel.__all__)
+
+from .parameter import *
+from .in_parallel import *
\ No newline at end of file
