@@ -7,21 +7,18 @@
 # for license details.
 """
 """
-from . import heat_exchange
 from . import bounded_numerical_specification
-from . import stream_mass_balance
+from . import reactor_specification
 from . import unit_group
 from . import utils
 
-__all__ = (*heat_exchange.__all__,
-           *bounded_numerical_specification.__all__,
-           *stream_mass_balance.__all__,
+__all__ = (*bounded_numerical_specification.__all__,
+           *reactor_specification.__all__,
            *unit_group.__all__,
-           *utils.__all__
+           *utils.__all__,
 )
 
-from .heat_exchange import *
 from .bounded_numerical_specification import *
-from .stream_mass_balance import *
+from .reactor_specification import *
 from .unit_group import *
 from .utils import *
\ No newline at end of file
