@@ -8,7 +8,6 @@
 """
 """
 import numpy as np
-from ._hx import HXutility
 from .. import Unit
 from .design_tools import size_batch
 from .decorators import cost
@@ -35,7 +34,7 @@ class BatchBioreactor(Unit, isabstract=True):
     
     **Abstract methods**
         
-    kinetic_model(z, t, *kinetic_constants) : 
+    kinetic_model(z, t, *kinetic_constants): 
         A staticmethod that returns effluent concentrations (z_t; kg/m^3)
         given the starting concentration (z; kg/m^3), reaction time (t; hr),
         and kinetic constants.
@@ -74,13 +73,13 @@ class BatchBioreactor(Unit, isabstract=True):
     
     References
     ----------
-    .. [5] D. Humbird, R. Davis, L. Tao, C. Kinchin, D. Hsu, and A.
-    Aden National. Renewable Energy Laboratory Golden, Colorado. P.
-    Schoen, J. Lukas, B. Olthof, M. Worley, D. Sexton, and D. Dudgeon.
-    Harris Group Inc. Seattle, Washington and Atlanta, Georgia. Process 
-    Design and Economics for Biochemical Conversion of Lignocellulosic
-    Biomass to Ethanol Dilute-Acid Pretreatment and Enzymatic Hydrolysis 
-    of Corn Stover. May 2011. Technical Report NREL/TP-5100-47764
+    .. [1] D. Humbird, R. Davis, L. Tao, C. Kinchin, D. Hsu, and A.
+        Aden National. Renewable Energy Laboratory Golden, Colorado. P.
+        Schoen, J. Lukas, B. Olthof, M. Worley, D. Sexton, and D. Dudgeon.
+        Harris Group Inc. Seattle, Washington and Atlanta, Georgia. Process 
+        Design and Economics for Biochemical Conversion of Lignocellulosic
+        Biomass to Ethanol Dilute-Acid Pretreatment and Enzymatic Hydrolysis 
+        of Corn Stover. May 2011. Technical Report NREL/TP-5100-47764
     
     """
     _units = {'Reactor volume': 'm3',
@@ -195,7 +194,7 @@ class BatchBioreactor(Unit, isabstract=True):
         elif self.V:
             f = lambda N: v_0 / N / V_wf * (tau + tau_0) / (1 - 1 / N) - self.V
             N = flx.IQ_interpolation(f, self.Nmin, self.Nmax,
-                                     xtol=0.01, ytol=1)
+                                     xtol=0.01, ytol=0.5, checkbounds=False)
             N = ceil(N)
         else:
             N = self._N
