@@ -61,10 +61,10 @@ class Fermentation(BatchBioreactor):
     Simulate a Fermentation object which models batch fermentation for the
     production of 1st generation ethanol using yeast.
     
-    >>> from biorefineries.lipidcane.chemicals import ethanol_chemicals 
+    >>> from biorefineries.lipidcane import chemicals
     >>> from biosteam.units import Fermentation
     >>> from biosteam import Stream, settings
-    >>> settings.set_thermo(ethanol_chemicals)
+    >>> settings.set_thermo(chemicals)
     >>> feed = Stream('feed',
     ...               Water=1.20e+05,
     ...               Glucose=1.89e+03,
@@ -84,70 +84,70 @@ class Fermentation(BatchBioreactor):
         flow (kmol/hr): Water     6.66e+03
                         Glucose   10.5
                         Sucrose   62.5
-                        DryYeast  1.03e+04
+                        Yeast  1.03e+04
     [1] missing stream
     outs...
     [0] CO2
         phase: 'g', T: 305.15 K, P: 101325 Pa
         flow (kmol/hr): Water    2.5
-                        CO2      244
                         Ethanol  0.582
+                        CO2      244
     [1] product
         phase: 'l', T: 305.15 K, P: 101325 Pa
         flow (kmol/hr): Water     6.6e+03
                         Ethanol   243
                         Glucose   13.6
-                        DryYeast  1.03e+04
+                        Yeast  1.03e+04
     >>> F1.results()
     Fermentation                                       Units        F1
     Power               Rate                              kW      11.6
                         Cost                          USD/hr     0.908
-    Chilled water       Duty                           kJ/hr -1.03e+07
-                        Flow                         kmol/hr  6.87e+03
-                        Cost                          USD/hr      51.3
+    Chilled water       Duty                           kJ/hr -1.05e+07
+                        Flow                         kmol/hr  7.06e+03
+                        Cost                          USD/hr      52.7
     Design              Reactor volume                    m3       246
                         Batch time                        hr      12.6
                         Loading time                      hr      1.57
                         Number of reactors                           8
-                        Reactor duty                   kJ/hr  1.03e+07
+                        Reactor duty                   kJ/hr  1.05e+07
                         Cleaning and unloading time       hr         3
                         Working volume fraction                    0.9
-    Purchase cost       Heat exchangers                  USD  4.09e+04
-                        Reactors                         USD  1.87e+06
+    Purchase cost       Heat exchangers                  USD  4.17e+04
+                        Reactors                         USD  2.08e+06
                         Agitators                        USD  1.16e+05
                         Cleaning in place                USD  8.87e+04
-    Total purchase cost                                  USD  2.12e+06
-    Utility cost                                      USD/hr      52.2
+    Total purchase cost                                  USD  2.32e+06
+    Utility cost                                      USD/hr      53.6
     
     References
     ----------
     .. [1] Oliveira, Samuel C., et al. "Discrimination between ethanol 
-    inhibition models in a continuous alcoholic fermentation process using
-    flocculating yeast." Applied biochemistry and biotechnology 74.3 (1998): 161-172.
+        inhibition models in a continuous alcoholic fermentation process using
+        flocculating yeast." Applied biochemistry and biotechnology 74.3 (1998): 161-172.
     
     .. [2] Oliveira, Samuel C., et al. "Continuous ethanol fermentation in a
-    tower reactor with flocculating yeast recycle: scale-up effects on process
-    performance, kinetic parameters and model predictions." Bioprocess
-    Engineering 20.6 (1999): 525-530.
+        tower reactor with flocculating yeast recycle: scale-up effects on process
+        performance, kinetic parameters and model predictions." Bioprocess
+        Engineering 20.6 (1999): 525-530.
     
     .. [3] Oliveira, Samuel C., et al. "Mathematical modeling of a continuous
-    alcoholic fermentation process in a two-stage tower reactor cascade with
-    flocculating yeast recycle." Bioprocess and biosystems engineering 38.3
-    (2015): 469-479.
+        alcoholic fermentation process in a two-stage tower reactor cascade with
+        flocculating yeast recycle." Bioprocess and biosystems engineering 38.3
+        (2015): 469-479.
     
     .. [4] Oliveira, Samuel C., et al. "Kinetic Modeling of 1‐G Ethanol
-    Fermentations." Fermentation Processes. InTech, 2017.
+        Fermentations." Fermentation Processes. InTech, 2017.
     
     .. [5] D. Humbird, R. Davis, L. Tao, C. Kinchin, D. Hsu, and A. Aden
-    National. Renewable Energy Laboratory Golden, Colorado. P. Schoen,
-    J. Lukas, B. Olthof, M. Worley, D. Sexton, and D. Dudgeon. Harris Group
-    Inc. Seattle, Washington and Atlanta, Georgia. Process Design and Economics
-    for Biochemical Conversion of Lignocellulosic Biomass to Ethanol Dilute-Acid
-    Pretreatment and Enzymatic Hydrolysis of Corn Stover. May 2011. Technical
-    Report NREL/TP-5100-47764
+        National. Renewable Energy Laboratory Golden, Colorado. P. Schoen,
+        J. Lukas, B. Olthof, M. Worley, D. Sexton, and D. Dudgeon. Harris Group
+        Inc. Seattle, Washington and Atlanta, Georgia. Process Design and Economics
+        for Biochemical Conversion of Lignocellulosic Biomass to Ethanol Dilute-Acid
+        Pretreatment and Enzymatic Hydrolysis of Corn Stover. May 2011. Technical
+        Report NREL/TP-5100-47764
     
     """
-    line = 'Fermentation'    
+    line = 'Fermentation'
     
     #: tuple[float] Kinetic parameters for the kinetic model. Default constants are fitted for Oliveria's model (mu_m1, mu_m2, Ks1, Ks2, Pm1, Pm2, Xm, Y_PS, a)
     kinetic_constants = (0.31,  # mu_m1
@@ -165,12 +165,12 @@ class Fermentation(BatchBioreactor):
                  efficiency=0.9, iskinetic=False):
         BatchBioreactor.__init__(self, ID, ins, outs, thermo,
                                  tau=tau, N=N, V=V, T=T, P=P, Nmin=Nmin, Nmax=Nmax)
-        self.hydrolysis = Reaction('Sucrose + Water -> 2Glucose', 'Sucrose', 1.00)
-        self.fermentation = Reaction('Glucose -> 2Ethanol + 2CO2',  'Glucose', efficiency)
+        self.hydrolysis_reaction = Reaction('Sucrose + Water -> 2Glucose', 'Sucrose', 1.00)
+        self.fermentation_reaction = Reaction('Glucose -> 2Ethanol + 2CO2',  'Glucose', efficiency)
         self.iskinetic = iskinetic
         self.efficiency = efficiency
         
-    def _calc_efficiency(self, feed, tau):
+    def _calc_efficiency(self, feed, tau): # pragma: no cover
         # Get initial concentrations
         y, e, s, w = feed.indices(['Yeast',
                                    '64-17-5',
@@ -198,7 +198,7 @@ class Fermentation(BatchBioreactor):
         return eff
         
     @staticmethod
-    def kinetic_model(z, t, *kinetic_constants):
+    def kinetic_model(z, t, *kinetic_constants): # pragma: no cover
         """
         Return change of yeast, ethanol, and substrate concentration in kg/m3.
         
@@ -242,17 +242,17 @@ class Fermentation(BatchBioreactor):
 
     @property
     def efficiency(self):
-        return self.fermentation.X
+        return self.fermentation_reaction.X
     @efficiency.setter
     def efficiency(self, efficiency):
-        self.fermentation.X = efficiency
+        self.fermentation_reaction.X = efficiency
 
     def _run(self):
         vent, effluent = self.outs
         effluent.mix_from(self.ins)
         effluent_mol = effluent.mol
-        self.hydrolysis(effluent_mol)
+        self.hydrolysis_reaction(effluent_mol)
         if self.iskinetic:
-            self.fermentation.X = self._calc_efficiency(effluent, self._tau)
-        self.fermentation(effluent_mol)
+            self.fermentation_reaction.X = self._calc_efficiency(effluent, self._tau)
+        self.fermentation_reaction(effluent_mol)
         vent.receive_vent(effluent)
