@@ -11,7 +11,7 @@ from .._unit import Unit
 from .._graphics import junction_graphics
 from .._power_utility import PowerUtility
 from thermosteam import MultiStream
-from ..utils.piping import Ins, Outs
+from ..utils.piping import Inlets, Outlets
 
 __all__ = ('Junction',)
 
@@ -22,7 +22,8 @@ chemicals_in_common = lambda upstream, downstream: \
     tuple(set(upstream.chemicals.IDs).intersection(downstream.chemicals.IDs))
 
 class Junction(Unit):
-    """Create a Junction object that copies specifications from `upstream`
+    """
+    Create a Junction object that copies specifications from `upstream`
     to `downstream`. This serves to connect streams with different
     Species object.
     
@@ -42,9 +43,9 @@ class Junction(Unit):
     Create a Junction object and connect streams with different Species objects:
         
     >>> from biosteam import *
-    >>> settings.thermo = Thermo(['Water'])
+    >>> settings.set_thermo(['Water'])
     >>> s1 = Stream('s1', Water=20)
-    >>> settings.thermo = Thermo(['Ethanol', 'Water'])
+    >>> settings.set_thermo(['Ethanol', 'Water'], cache=True)
     >>> s2 = Stream('s2') # Note that s2 and s1 have different chemicals defined
     >>> J1 = units.Junction('J1', s1, s2)
     >>> J1.simulate()
@@ -60,16 +61,16 @@ class Junction(Unit):
         flow (kmol/hr): Water  20
         
     """
+    _stacklevel = Unit._stacklevel
     _graphics = junction_graphics
-    _N_ins = _N_outs = 1
     heat_utilities = ()
     power_utility = PowerUtility()
     def __init__(self, ID="", upstream=None, downstream=None, thermo=None):
         thermo = self._load_thermo(thermo)
         self._specification = None
         self._chemicals_in_common = self._past_streams = ()
-        self._ins = Ins(self, self._N_ins, upstream, thermo)
-        self._outs = Outs(self, self._N_outs, downstream, thermo)
+        self._ins = Inlets(self, 1, upstream, thermo, True, self._stacklevel)
+        self._outs = Outlets(self, 1, downstream, thermo, True, self._stacklevel)
         self._register(ID)
     
     def set_spec(self, *args, **kwargs):
