@@ -1,13 +1,18 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Mon Mar  4 16:40:50 2019
-
-@author: yoelr
 """
 from .decorators import cost
-from ..utils.design_warning import lb_warning
+from ..utils.unit_warnings import lb_warning
 from .._unit import Unit
 
+__all__ = ('ScrewFeeder',)
+
 @cost('Flow rate', ub=10e4, CE=567, cost=1096, n=0.22)
 class ScrewFeeder(Unit):
     length = 30 #: ft
@@ -24,5 +29,5 @@ class ScrewFeeder(Unit):
         if F_vol < self._minimum_flow:
             lb_warning('Flow rate', F_vol, self._minimum_flow)
         Design['Flow rate'] = F_vol
-        F_mass = feed.F_mass*0.0006124 #lb/s
+        F_mass = 0.0006124 * feed.F_mass # lb/s
         self._power_utility(0.0146*F_mass**0.85*self.length*0.7457)
\ No newline at end of file
