@@ -1,8 +1,11 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Wed Jul 17 01:11:46 2019
-
-@author: yoelr
 """
 from .decorators import cost
 from .. import Unit
@@ -14,8 +17,8 @@ __all__ = ('VentScrubber',)
 class VentScrubber(Unit): 
     _N_ins = _N_outs = 2
     _units = {'Flow rate': 'kg/hr'}
-    def __init__(self, ID='', ins=None, outs=(), *, gas):
-        Unit.__init__(self, ID, ins, outs)
+    def __init__(self, ID='', ins=None, outs=(), thermo=None, *, gas):
+        Unit.__init__(self, ID, ins, outs, thermo)
         self.gas = gas
     
     def _run(self):
