@@ -1,13 +1,18 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Thu Jun  6 01:25:26 2019
+Cost indices.
 
-@author: yoelr
 """
 
 __all__ = ('CEPCI_by_year',)
 
-#: dict[int: int] 
+#: Chemical Engineering Plant Cost Index by year
 CEPCI_by_year = {1980: 261,
                  1981: 297,
                  1982: 314,
@@ -41,4 +46,8 @@ CEPCI_by_year = {1980: 261,
                  2010: 551,
                  2011: 586,
                  2012: 585,
-                 2013: 567}
\ No newline at end of file
+                 2013: 567,
+                 2014: 576,
+                 2015: 542,
+                 2016: 568,
+                 2018: 603}
\ No newline at end of file
