@@ -11,12 +11,22 @@ from flexsolve import njitable
 from math import pi
 
 __all__ = ('cylinder_diameter_from_volume', 
-           'cylinder_volume_from_diameter')
+           'cylinder_volume_from_diameter',
+           'cylinder_area',
+           'circumference')
 
 @njitable(cache=True)
 def cylinder_diameter_from_volume(volume, length_to_diameter):
-    return (4. * length_to_diameter * volume / pi)**(1./3.)
+    return (4. * volume / pi / length_to_diameter)**(1./3.)
 
 @njitable(cache=True)
 def cylinder_volume_from_diameter(diameter, length_to_diameter):
-    return pi * (diameter / 2) ** 2 * diameter / length_to_diameter
\ No newline at end of file
+    return pi * (diameter / 2) ** 2 * diameter * length_to_diameter
+
+@njitable(cache=True)
+def cylinder_area(diameter, length):
+    return circumference(diameter) * length
+
+@njitable(cache=True)
+def circumference(diameter):
+    return pi * diameter
\ No newline at end of file
