@@ -9,25 +9,34 @@
 """
 
 from ..._facility import Facility
+from ._chemical_capital_investment import *
 from ._boiler_turbogenerator import *
 from ._cooling_tower import *
 from ._chilled_water_package import *
 from ._process_water_center import *
 from ._air_distribution_package import *
 from ._blowdown_mixer import *
+from ._cleaning_in_place import *
+from .hxn import *
 
+from . import _chemical_capital_investment
 from . import _blowdown_mixer
 from . import _boiler_turbogenerator
 from . import _cooling_tower
 from . import _chilled_water_package 
 from . import _process_water_center
 from . import _air_distribution_package
+from . import _cleaning_in_place
+from . import hxn
 
 __all__ = ('Facility',
+           *_chemical_capital_investment.__all__,
            *_blowdown_mixer.__all__,
            *_air_distribution_package.__all__,
            *_cooling_tower.__all__,
            *_boiler_turbogenerator.__all__,
            *_chilled_water_package.__all__,
-           *_process_water_center.__all__
+           *_process_water_center.__all__,
+           *_cleaning_in_place.__all__,
+           *hxn.__all__,
 )
