@@ -8,17 +8,11 @@
 """
 """
 from . import Facility
+from .. import Mixer
 from ..._graphics import mixer_graphics
 
 __all__ = ('BlowdownMixer',)
 
-class BlowdownMixer(Facility):
+class BlowdownMixer(Facility, Mixer):
     network_priority = 2
-    _graphics = mixer_graphics
-    _N_outs = 1
-    _N_ins = 2
-    _ins_size_is_fixed = False
-    
-    def _run(self):
-        s_out, = self.outs
-        s_out.mix_from(self.ins)
\ No newline at end of file
+    _graphics = mixer_graphics
\ No newline at end of file
