@@ -26,7 +26,7 @@ __all__ = ('BoilerTurbogenerator',)
 @cost('Flow rate', 'Deaerator',
       CE=551, cost=305e3, S=235803, n=0.6, BM=3.0)
 @cost('Flow rate', 'Boiler',
-      CE=551, cost=28550e3, kW=1000, S=238686, n=0.6, BM=1.8)
+      CE=551, cost=28550e3, kW=1371, S=238686, n=0.6, BM=1.8)
 class BoilerTurbogenerator(Facility):
     """
     Create a BoilerTurbogenerator object that will calculate electricity
@@ -46,16 +46,32 @@ class BoilerTurbogenerator(Facility):
         
         [3] Natural gas to satisfy steam and power requirement.
         
+        [4] Lime for flue gas desulfurization.
+        
+        [5] Boiler chemicals.
+        
     outs : stream sequence
         [0] Total emissions produced.
         
         [1] Blowdown water.
+        
+        [2] Ash disposal.
+        
     boiler_efficiency : float
         Fraction of heat transfered to steam.
     turbo_generator_efficiency : float
         Fraction of steam heat converted to electricity.
-    agent : UtilityAgent
-        Steam produced.
+    agent : UtilityAgent, optional
+        Steam produced. Defaults to low pressure steam.
+    other_agents = () : Iterable[UtilityAgent]
+        Other steams produced.
+    natural_gas_price = 0.218 : float
+        Price of natural gas [USD/kg].
+    
+    Notes
+    -----
+    The flow rate of natural gas, lime, and boiler chemicals (streams 3-5)
+    is set by the BoilerTurbogenerator object during simulation.
     
     References
     ----------
@@ -69,26 +85,26 @@ class BoilerTurbogenerator(Facility):
     network_priority = 0
     boiler_blowdown = 0.03
     RO_rejection = 0
-    _N_ins = 4
-    _N_outs = 2
+    _N_ins = 6
+    _N_outs = 3
     _N_heat_utilities = 0
     _units = {'Flow rate': 'kg/hr',
               'Work': 'kW'}
     
-    def __init__(self, ID='', ins=None, outs=(), thermo=None, *,
+    def __init__(self, ID='', ins=None, 
+                 outs=('emissions',
+                       'rejected_water_and_blowdown',
+                       'ash_disposal'),
+                 thermo=None, *,
                  boiler_efficiency=0.80,
                  turbogenerator_efficiency=0.85,
                  side_steam=None,
-                 agent=HeatUtility.get_heating_agent('low_pressure_steam'),
-                 other_agents = ()):
+                 agent=None,
+                 other_agents = (),
+                 natural_gas_price=0.218):
         Facility.__init__(self, ID, ins, outs, thermo)
-        self.agent = agent
-        self.makeup_water = makeup_water = agent.to_stream('boiler_makeup_water')
-        loss = makeup_water.flow_proxy()
-        loss.ID = 'rejected_water_and_blowdown'
-        self.ins[-1] = bst.Stream(None, thermo=self.thermo, phase='g')
-        self.ins[-2] = makeup_water
-        self.outs[-1] = loss
+        self.agent = agent = agent or HeatUtility.get_heating_agent('low_pressure_steam')
+        self.natural_gas_price = natural_gas_price
         self.boiler_efficiency = boiler_efficiency
         self.turbogenerator_efficiency = turbogenerator_efficiency
         self.steam_utilities = set()
@@ -97,6 +113,20 @@ class BoilerTurbogenerator(Facility):
         self.side_steam = side_steam
         self.other_agents = other_agents
     
+    @property
+    def makeup_water(self):
+        """[Stream] Makeup water due to boiler blowdown."""
+        return self.ins[2]
+    
+    @property
+    def natural_gas(self):
+        """[Stream] Natural gas to satisfy steam and electricity requirements."""
+        return self.ins[3]
+    
+    @property
+    def utility_cost(self):
+        return super().utility_cost + self.natural_gas_price * self.natural_gas.F_mass
+    
     def _run(self): pass
 
     def _load_utility_agents(self):
@@ -119,18 +149,21 @@ class BoilerTurbogenerator(Facility):
         Design = self.design_results
         self._load_utility_agents()
         mol_steam = sum([i.flow for i in self.steam_utilities])
-        feed_solids, feed_gas, makeup_water, feed_CH4 = self.ins
-        emissions, _ = self.outs
+        feed_solids, feed_gas, makeup_water, feed_CH4, lime, chems = self.ins
+        emissions, blowdown_water, ash_disposal = self.outs
+        if not ash_disposal.price: 
+            ash_disposal.price = -0.031812704433277834
+        if not lime.price:
+            lime.price = 0.19937504680689402
+        if not chems.price:
+            chems.price = 4.995862254032183
         H_steam =  sum([i.duty for i in self.steam_utilities])
         side_steam = self.side_steam
         if side_steam: 
             H_steam += side_steam.H
             mol_steam += side_steam.F_mol
         steam_demand.imol['7732-18-5'] = mol_steam 
-        if not mol_steam:
-            duty_over_mol = self.agent.Hvap
-        else:
-            duty_over_mol = H_steam / mol_steam
+        duty_over_mol = 39000 # kJ / mol-superheated steam 
         emissions_mol = emissions.mol
         emissions.T = self.agent.T
         emissions.P = 101325
@@ -154,7 +187,7 @@ class BoilerTurbogenerator(Facility):
             emissions.imol['O2'] = 0
             H_content = B_eff * H_combustion - emissions.H
             
-            #: [float] Total steam demand produced by the boiler (kmol/hr)
+            #: [float] Total steam produced by the boiler (kmol/hr)
             self.total_steam = H_content / duty_over_mol 
             Design['Flow rate'] = flow_rate = self.total_steam * 18.01528
             
@@ -175,18 +208,29 @@ class BoilerTurbogenerator(Facility):
         excess_electricity = calculate_excess_electricity_at_natual_gas_flow(0)
         if excess_electricity < 0:
             f = calculate_excess_electricity_at_natual_gas_flow
-            flx.aitken_secant(f, 10, 20, xtol=1, ytol=1)
+            lb = excess_electricity * 3600 / feed_CH4.chemicals.CH4.LHV
+            ub = 10 * lb
+            while f(ub) < 0.: 
+                lb = ub
+                ub *= 2
+            flx.IQ_interpolation(f, lb, ub, xtol=1, ytol=1)
         
         hu_cooling = bst.HeatUtility()
         hu_cooling(self.cooling_duty, steam_demand.T)
         hus_heating = bst.HeatUtility.sum_by_agent(tuple(self.steam_utilities))
         for hu in hus_heating: hu.reverse()
         self.heat_utilities = (*hus_heating, hu_cooling)
-        
-        makeup_water.imol['7732-18-5'] = (
+        water_index = self.chemicals.index('7732-18-5')
+        blowdown_water.mol[water_index] = makeup_water.mol[water_index] = (
                 self.total_steam * self.boiler_blowdown * 1/(1-self.RO_rejection)
         )
-
+        ash_index = self.chemicals.index('Ash')
+        ash_disposal.mol[ash_index] = F_mass_ash = emissions.mol[ash_index]
+        lime.mol[ash_index] = F_mass_lime = 0.21 * F_mass_ash
+        chems.mol[ash_index] = F_mass_chems = 0.0002846242774566474 * F_mass_lime
+        ash_disposal.mol[ash_index] += F_mass_chems +  F_mass_lime
+        emissions.mol[ash_index] = 0.
+        
     def _cost(self):
         self._decorated_cost()
         self.power_utility.production = self.design_results['Work']
