@@ -7,23 +7,25 @@
 # for license details.
 """
 """
-from thermosteam.utils import colors
+from thermosteam import utils
 from . import (misc,
-               plotting,
+               patches,
                tictoc,
                not_implemented_method,
                piping,
                stream_link_options,
                unit_warnings,
                functors,
+               stream_filters,
 )
 __all__ = ('colors',
            'misc', 
-           'plotting', 
+           'patches', 
            'tictoc',
            'not_implemented_method',
            'functors',
-           *plotting.__all__, 
+           *utils.__all__,
+           *patches.__all__, 
            *not_implemented_method.__all__, 
            *misc.__all__, 
            *tictoc.__all__, 
@@ -31,12 +33,17 @@ __all__ = ('colors',
            *stream_link_options.__all__,
            *unit_warnings.__all__,
            *functors.__all__,
+           *stream_filters.__all__,
 )
+from thermosteam.utils import *
 from .not_implemented_method import *
 from .misc import *
-from .plotting import *
+from .patches import *
 from .tictoc import *
 from .piping import *
 from .stream_link_options import *
 from .unit_warnings import *
-from .functors import *
\ No newline at end of file
+from .functors import *
+from .stream_filters import *
+
+del utils
\ No newline at end of file
