@@ -1,11 +1,13 @@
 # -*- coding: utf-8 -*-
+# BioSTEAM: The Biorefinery Simulation and Techno-Economic Analysis Modules
+# Copyright (C) 2020, Yoel Cortes-Pena <yoelcortes@gmail.com>
+# 
+# This module is under the UIUC open-source license. See 
+# github.com/BioSTEAMDevelopmentGroup/biosteam/blob/master/LICENSE.txt
+# for license details.
 """
-Created on Fri Dec 27 00:45:47 2019
-
-@author: yoelr
 """
 from collections import namedtuple
-from .not_implemented_method import NotImplementedMethod
 
 __all__ = ('StreamLinkOptions',
            'static',
@@ -24,7 +26,6 @@ static_flow_link_options = StreamLinkOptions(flow=True, TP=False, phase=False)
 
 def static(cls):
     cls._stream_link_options = static_link_options
-    cls._run = NotImplementedMethod
     cls._N_ins = cls._N_outs = 1
     return cls
 
